<?php

namespace ShopEngine\Core;

use ShopEngine\Traits\Singleton;

class Query_Modifier
{

	use Singleton;

	public function init() {

		add_action('pre_get_posts', [$this, 'modify_query']);
	}

	public function modify_query($query) {

		if(is_admin() || !$query->is_main_query() || $query->is_single === true) {
			return;
		}

		if(!isset($query->query_vars['wc_query']) || $query->query_vars['wc_query'] != 'product_query') {
			return;
		}

		// query filter begins

		// update query for product per page filter
		if(!empty($_GET['shopengine_products_per_page'])) {
			$query->set('posts_per_page', absint($_GET['shopengine_products_per_page']));
		}

		//update query if query param has color filter
		foreach($_GET as $key => $vals) {

			$prefix = 'shopengine_filter_color_';

			if(strpos($key, $prefix) !== false) {

				$tax = str_replace($prefix, '', $key);

				$query->set(
					'tax_query',
					[
						'relation' => 'OR',
						[
							'taxonomy' => $tax,
							'field'    => 'slug',
							'terms'    => $vals,
							'operator' => 'IN',
						],
					]
				);
			}
		}

		//update query if query param has size filter
		if(!empty($_GET['shopengine_filter_size'])) {

			$size_values = explode(',', $_GET['shopengine_filter_size']);

			if( is_array($size_values) && !empty($size_values) ){
				$query->set('tax_query', [
					[
						'taxonomy'        => 'pa_size',
						'field'           => 'slug',
						'terms'           =>  $size_values,
						'operator'        => 'IN',
					]
				]);
			}
		}

		//update query if query param has category filter
		if(!empty($_GET['shopengine_filter_category'])) {

			$category_values = explode(',', $_GET['shopengine_filter_category']);

			if( is_array($category_values) && !empty($category_values) ){
				$query->set('tax_query', [
					[
						'taxonomy'        => 'product_cat',
						'field'           => 'slug',
						'terms'           =>  $category_values,
						'operator'        => 'IN',
					]
				]);
			}
		}


		// query filter ends
	}
}
